#
# Use: The third arg, if given can be used to choose 32 bit archs over 64 bit when the 
# choice exists. Really a hold over from a long time ago.
#
# AC_CHECK_IDL(
#    [
#        LIBS="$LIBS $IDL_LIBS"
#        CFLAGS="$CFLAGS $IDL_CFLAGS"
#    ],
#    [
#        AC_MSG_ERROR([idl library and idl headers are required.])
#    ],
#    [ 0 ]
# )

AC_DEFUN([AC_CHECK_IDL],
[
    AC_ARG_WITH(idl,
        [  --with-idl=ARG       Where is the IDL root directory],
        IDL_ROOT=${withval}, IDL_ROOT="$IDL_ROOT")

    AC_REQUIRE([AC_CANONICAL_HOST])

    # IDL_DIR is the environment variable used for IDL. So, if IDL_ROOT is
    # not set then try using IDL_DIR.
    AC_MSG_CHECKING(for the IDL root directory)
    if test -z "$IDL_ROOT"
    then
	IDL_ROOT="$IDL_DIR"
	if test -z "$IDL_ROOT"
	then
     	    m4_if([$2], [], [:], [$2])
	fi
    fi

    AC_MSG_RESULT($IDL_ROOT)
    AC_SUBST(IDL_ROOT)

    IDL_PATH_INC="$IDL_ROOT/external"
    IDL_CFLAGS=
    ac_idl_h='no'
    ac_idl_save_CFLAGS="$CFLAGS"
    CFLAGS="$CFLAGS -I$IDL_PATH_INC"
    AC_CHECK_HEADERS([export.h],
      [ac_idl_h='yes'],
      [ac_idl_h='no']
    )
    CFLAGS=$ac_idl_save_CFLAGS

    AC_MSG_CHECKING(for the IDL include directory)
    if test x$ac_idl_h = xno; then
        m4_if([$2], [], [:], [$2])
    fi

    AS_IF([test x$ac_idl_h = xyes ],
      [IDL_CFLAGS="-I$IDL_PATH_INC"]
    )
    AC_MSG_RESULT($IDL_CFLAGS)
    AC_SUBST([IDL_CFLAGS])

    # Now find where the IDL 5.2 or later sharable libraries live.
    AC_MSG_CHECKING(for the IDL sharable library directory)
    # CAN I RUN AN OUTSIDE EXECUTABLE
    IDL_LIBS=""

    # Set PREFER_32 to make that the preference when a choice exists.
    # A hold over from the old days. We can make this a param to
    # configure and this defun
    if test -n "$3" -a "$3" = "1"; then
        PREFER_32=1
    else
	PREFER_32=0
    fi
    ac_idl_os=
    ac_idl_arch=
    ac_idl_uname=`uname`
    case $ac_idl_uname in
        "Darwin")
	    ac_idl_os="darwin"
	    ac_idl_arch=".x86_64" 
	    # You'd think that ARCH should be ".`arch`" but not with
	    # OS/X. From apple's help forum: The source of your
	    # confusion is the assumption that a 32-bit kernel will
	    # run 32-bit binaries. While it can, the default and
	    # proper behavior is to run applications in 64-bit
	    # (x86_64) on CPUs that support it. Only the very first
	    # series of Intel Macs (ca. 2006) used 32-bit CPUs. Any
	    # contemporary Mac should run 64-bit application binaries
	    # if available.
	    ;;

    	"SunOS")				# Solaris
	    ac_idl_os="solaris2"
            ac_idl_bin_arch=/usr/bin/arch
            if [ -x /usr/bin/arch ]; then
                ac_idl_bin_arch=/usr/bin/arch
            elif [ -x /usr/ucb/arch ]; then
                ac_idl_bin_arch=/usr/ucb/arch
             fi
             if [ `$ac_idl_bin_arch` = i86pc ]; then
                 ac_idl_arch=".x86"
	     else
	         # If the system is running the 64-bit SunOS kernel, a 64-bit
	  	 # IDL is installed, and the user did not specify the -32
	  	 # command argument, then run the 64-bit version. Otherwise
	  	 # the 32-bit version will work on all supported platforms.
          	 ac_idl_arch=".sparc"
	  	 if [ \( $PREFER_32 = 0 \) -a \( -x /bin/isainfo \) \
               	     -a \( -f $IDL_DIR/bin/bin.$ac_idl_os.sparc64/$APPLICATION \) ]; then
            	     if [ `/bin/isainfo -b` = 64 ]; then
	      	         ac_idl_arch=.sparc64
            	     fi
                 fi
             fi
	     ;;

	"AIX")
	    ac_idl_os="ibm"
	    if [ \( $PREFER_32 = 0 \) \
		-a \( -f $IDL_DIR/bin/bin.$ac_idl_os.rs6000_64/$APPLICATION \) ]; then
		ac_idl_arch=.rs6000_64
	    fi
	    ;;

	"HP-UX")
	    ac_idl_os="hp"
	    if [ \( $PREFER_32 = 0 \) -a \( -x /bin/getconf \) \
		-a \( -f $IDL_DIR/bin/bin.$ac_idl_os.pa64/$APPLICATION \) ]; then
		if [ `/bin/getconf KERNEL_BITS` = 64 ]; then
		  ac_idl_arch=.pa64
		fi
	    fi
	    ;;

	"IRIX"|"IRIX6"|"IRIX64")
	    # 32-bit kernels reply with IRIX. The other variants come from
	    # kernels that are capable of running 64-bit binaries.
	    ac_idl_os="sgi"
	    if [ \( $PREFER_32 = 0 \) -a \( "$ac_idl_uname" != "IRIX" \) \
		-a \( -f $IDL_DIR/bin/bin.$ac_idl_os.mips64/$APPLICATION \) ]; then
		ac_idl_arch=.mips64
	    fi
	    ;;

	"OSF1")
	    ac_idl_os="alpha"
	    ;;

	"Linux")
	    ac_idl_os="linux"
	    ac_idl_arch=".x86"
	    if [ -x /bin/arch ]; then
	      if [ `/bin/arch` = alpha ]; then
		ac_idl_arch=.alpha
	      fi
	    fi
	    ;;

	*)
	    ;;

    esac

    if test -n "${ac_idl_os}" -a -n "${ac_idl_arch}"; then
        IDL_LIBS="$IDL_ROOT/bin/bin.${ac_idl_os}${ac_idl_arch}"
	AC_MSG_RESULT($IDL_LIBS)
    	AC_SUBST(IDL_LIBS)
    	ac_libs_ok=yes
    else
	AC_MSG_RESULT([I could not find the IDL library directory (os: $ac_idl_os; arch: $ac_idl_arch.)])
     	ac_libs_ok=no
    fi

    AS_IF([test "$ac_libs_ok" = 'no' -o "$ac_idl_h" = 'no'],
      [m4_if([$2], [], [:], [$2])],
      [m4_if([$1], [], [:], [$1])]
    )
])

