; docformat = 'rst'

;+
; IDL-OPENDAP interface.
;
; :Private:
;
; :Author: 
;    Patrick West <mrpatrickwest@gmail.com> UCAR/HAO
;
;    Dan Holloway <d.holloway@opendap.org>
;
; :Copyright:
;    Copyright (c) 2006 OPeNDAP, Inc.
;
;    Based on IDL-DODS Written by Daniel J. Carr - Research Systems, Inc.
;
;    This is free software; you can redistribute it and/or
;    modify it under the terms of the GNU Lesser General Public
;    License as published by the Free Software Foundation; either
;    version 2.1 of the License, or (at your option) any later version.
; 
;    This library is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;    Lesser General Public License for more details.
; 
;    You should have received a copy of the GNU Lesser General Public
;    License along with this library; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;    You can contact OPeNDAP, Inc. at PO Box 112, Saunderstown, RI. 02874-0112.
;-

;+
; Function to establish a connection and get OPeNDAP data.
;
; :Returns:
;    Status (1=ok, 0=fail)
;
; :Params:
;    url : in, required, type=string
;       URL location to connect to
;    opendap_data : out, optional, type=any
;       Returns the requested OPeNDAP data as an IDL structure
;    debug : in, required, type=int
;       set to 1 to print debug statements
;
; :Keywords:
;    ce : in, optional, type=string
;       Data constraint expression
;    deflate : in, optional, type=boolean
;       ACCEPT_DEFLATE flag  (1=yes (default), 0=no)
;    seq_blocksize : in, optional, type=long, default=4096
;       How many elements should be allocated at once for sequence data;
;       default value is 4096
;    mode : in, required, type=ulong
;       mode flag for data request; 1 (dds), 4 (fulldds), 3 (fulldata)
;    indicate_type : in, optional, type=boolean
;       set to have each structure in the return value indicate it's type 
;       (i.e. "GRID", "GROUP", etc.)
;-
FUNCTION OPENDAP_GET_DATA, url, opendap_data, debug, CE=ce, DEFLATE=deflate, $
                           SEQ_BLOCKSIZE=seq_blocksize, MODE=mode, $
                           INDICATE_TYPE=indicateType
  COMPILE_OPT strictarr
     
  get_data = mode EQ 1UL OR mode EQ 4UL ? 0UL : 1UL
  root_node = 0UL

  conn = OPENDAP_CONNECT([BYTE(STRTRIM(url[0], 2)), 0B], $
                         [BYTE(STRTRIM(ce[0], 2)), 0B], $
                         root_node, $
                         ULONG(KEYWORD_SET(deflate)), $
                         ULONG(mode))
  
  IF ((conn EQ 0ul) OR (root_node EQ 0ul)) THEN BEGIN
    PRINT, 'failed to retrieve opendap data'
    RETURN, 0UL
  ENDIF
  
  ; Handle the root node
  status = OPENDAP_HANDLE_NODE(opendap_data, root_node, $
                               get_data, debug, INDICATE_TYPE=indicateType)

  RETURN, status
END





