#!/bin/sh

IDL_ROOT=$1
IDL_DIR=$1

PREFER_32=0
for arg
do
  if [ "$arg" = "-32" ]; then
    PREFER_32=1
  fi
done

OS=
ARCH=
UNAME=`uname`
case $UNAME in

    "Darwin")
        OS="darwin"
	ARCH=".x86_64" 
	# You'd think that ARCH should be ".`arch`" but not with OS/X. From
	# apple's help forum: The source of your confusion is the
	# assumption that a 32-bit kernel will run 32-bit binaries.
	# While it can, the default and proper behavior is to run
	# applications in 64-bit (x86_64) on CPUs that support it.
	# Only the very first series of Intel Macs (ca. 2006) used
	# 32-bit CPUs. Any contemporary Mac should run 64-bit
	# application binaries if available.
	;;

    "SunOS")				# Solaris
	OS="solaris2"
        BIN_ARCH=/usr/bin/arch
        if [ -x /usr/bin/arch ]; then
            BIN_ARCH=/usr/bin/arch
        elif [ -x /usr/ucb/arch ]; then
            BIN_ARCH=/usr/ucb/arch
        fi
        if [ `$BIN_ARCH` = i86pc ]; then
          ARCH=".x86"
	else
	  # If the system is running the 64-bit SunOS kernel, a 64-bit
	  # IDL is installed, and the user did not specify the -32
	  # command argument, then run the 64-bit version. Otherwise
	  # the 32-bit version will work on all supported platforms.
          ARCH=".sparc"
	  if [ \( $PREFER_32 = 0 \) -a \( -x /bin/isainfo \) \
               -a \( -f $IDL_DIR/bin/bin.$OS.sparc64/$APPLICATION \) ]; then
            if [ `/bin/isainfo -b` = 64 ]; then
	      ARCH=.sparc64
            fi
          fi
        fi
	;;

    "AIX")
	OS="ibm"
	if [ \( $PREFER_32 = 0 \) \
            -a \( -f $IDL_DIR/bin/bin.$OS.rs6000_64/$APPLICATION \) ]; then
	    ARCH=.rs6000_64
        fi
	;;

    "HP-UX")
	OS="hp"
	if [ \( $PREFER_32 = 0 \) -a \( -x /bin/getconf \) \
            -a \( -f $IDL_DIR/bin/bin.$OS.pa64/$APPLICATION \) ]; then
            if [ `/bin/getconf KERNEL_BITS` = 64 ]; then
	      ARCH=.pa64
            fi
	fi
	;;

    "IRIX"|"IRIX6"|"IRIX64")
        # 32-bit kernels reply with IRIX. The other variants come from
        # kernels that are capable of running 64-bit binaries.
	OS="sgi"
	if [ \( $PREFER_32 = 0 \) -a \( "$UNAME" != "IRIX" \) \
            -a \( -f $IDL_DIR/bin/bin.$OS.mips64/$APPLICATION \) ]; then
	    ARCH=.mips64
        fi
	;;

    "OSF1")
	OS="alpha"
	;;

    "Linux")
	OS="linux"
        ARCH=".x86"
        if [ -x /bin/arch ]; then
          if [ `/bin/arch` = alpha ]; then
            ARCH=.alpha
          fi
        fi
	;;
		
    *)
	echo "$APPLICATION: Unable to recognize system architecture."
	exit 1
	;;

esac

echo "${IDL_ROOT}/bin/bin.${OS}${ARCH}"
